@@ -1,14 +1,19 @@
 #! perl
 
+package main;
+
+our $config;
+
 package App::Music::ChordPro::Output::PDF::Writer;
 
 use strict;
 use warnings;
 use Encode;
-use PDF::API2;
 use Text::Layout;
 use IO::String;
+use Carp;
 
+use App::Music::ChordPro::Utils qw( expand_tilde );
 use App::Music::ChordPro::Output::Common qw( fmt_subst prep_outlines demarkup );
 
 # For regression testing, run perl with PERL_HASH_SEED set to zero.
@@ -22,12 +27,16 @@ my %fontcache;			# speeds up 2 seconds per song
 sub new {
     my ( $pkg, $ps, $pdfapi ) = @_;
     my $self = bless { ps => $ps }, $pkg;
+    $self->{pdfapi} = $pdfapi;
     $self->{pdf} = $pdfapi->new;
     $self->{pdf}->{forcecompress} = 0 if $regtest;
     $self->{pdf}->mediabox( $ps->{papersize}->[0],
 			    $ps->{papersize}->[1] );
     $self->{layout} = Text::Layout->new( $self->{pdf} );
-    %fontcache = () if $::__EMBEDDED__;
+    $self->{tmplayout} = undef;
+
+    %fontcache = ();
+
     $self;
 }
 
@@ -36,13 +45,19 @@ sub info {
     unless ( $info{CreationDate} ) {
 	my @tm = gmtime( $regtest ? $faketime : time );
 	$info{CreationDate} =
-	  sprintf("D:%04d%02d%02d%02d%02d%02d+00'00'",
+	  sprintf("D:%04d%02d%02d%02d%02d%02d+01'00",
 		  1900+$tm[5], 1+$tm[4], @tm[3,2,1,0]);
     }
-    $self->{pdf}->info( %info );
+    if ( $self->{pdf}->can("info_metadata") ) {
+	for ( keys(%info) ) {
+	    $self->{pdf}->info_metadata( $_, $info{$_} );
+	}
+    }
+    else {
+	$self->{pdf}->info(%info);
+    }
 }
 
-
 sub wrap {
     my ( $self, $text, $m ) = @_;
 
@@ -65,6 +80,41 @@ sub wrap {
     return ( $text, $ex );
 }
 
+sub _fgcolor {
+    my ( $self, $col ) = @_;
+    if ( !defined($col) || $col =~ /^foreground(?:-medium|-light)?$/ ) {
+	$col = $self->{ps}->{theme}->{$col//"foreground"};
+    }
+    elsif ( $col eq "background" ) {
+	$col = $self->{ps}->{theme}->{background};
+    }
+    elsif ( !$col ) {
+	Carp::confess("Undefined fgcolor: $col");
+    }
+    $col;
+}
+
+sub _bgcolor {
+    my ( $self, $col ) = @_;
+    if ( !defined($col) || $col eq "background" ) {
+	$col = $self->{ps}->{theme}->{background};
+    }
+    elsif ( $col =~ /^foreground(?:-medium|-light)?$/ ) {
+	$col = $self->{ps}->{theme}->{$col};
+    }
+    elsif ( !$col ) {
+	Carp::confess("Undefined bgcolor: $col");
+    }
+    $col;
+}
+
+sub _yflip {
+    #warn("Text::Layout = $Text::Layout::VERSION\n" );
+    $Text::Layout::VERSION gt "0.027";
+}
+
+my $yflip;
+
 sub text {
     my ( $self, $text, $x, $y, $font, $size, $nomarkup ) = @_;
 #    print STDERR ("T: @_\n");
@@ -73,75 +123,47 @@ sub text {
 
     $self->{layout}->set_font_description($font->{fd});
     $self->{layout}->set_font_size($size);
-    if ( $font->{color} && $font->{color} ne "black" ) {
-	$text = "<span color='" . $font->{color} . "'>" . $text . "</span>";
-    }
+    # We don't have set_color in the API.
+    $self->{layout}->{_currentcolor} = $self->_fgcolor($font->{color});
+    # Watch out for regression... May have to do this in the nomarkup case only.
     if ( $nomarkup ) {
+	$text =~ s/'/\x{2019}/g;		# friendly quote
 	$self->{layout}->set_text($text);
     }
     else {
 	$self->{layout}->set_markup($text);
+	for ( @{ $self->{layout}->{_content} } ) {
+	    $_->{text} =~ s/\'/\x{2019}/g;	# friendly quote
+	}
     }
     $y -= $self->{layout}->get_baseline;
     $self->{layout}->show( $x, $y, $self->{pdftext} );
 
-    my $e = ($self->{layout}->get_pixel_extents)[1];
+    my $e = $self->{layout}->get_pixel_extents;
+    if ( ref($e) eq 'ARRAY' ) { # Text::Layout <= 0.026
+	$e = $e->[1];
+    }
+    elsif ( $yflip //= _yflip() ) {
+	$e->{y} += $e->{height};
+    }
 
     # Handle decorations (background, box).
-    my $bgcol = $font->{background};
+    my $bgcol = $self->_bgcolor($font->{background});
     undef $bgcol if $bgcol && $bgcol =~ /^no(?:ne)?$/i;
     my $debug = $ENV{CHORDPRO_DEBUG_TEXT} ? "magenta" : undef;
     my $frame = $font->{frame} || $debug;
     undef $frame if $frame && $frame =~ /^no(?:ne)?$/i;
     if ( $bgcol || $frame ) {
-	#printf("BB: %.2f %.2f %.2f %.2f\n", @{$e}{qw( x y width height ) } );
+	printf("BB: %.2f %.2f %.2f %.2f\n", @{$e}{qw( x y width height ) } )
+	  if $debug;
 	# Draw background and.or frame.
 	my $d = $debug ? 0 : 1;
-	$frame = $debug || $font->{color} || "black" if $frame;
+	$frame = $debug || $font->{color} || $self->{ps}->{theme}->{foreground} if $frame;
+	# $self->crosshair( $x, $y, 20, 0.2, "magenta" );
 	$self->rectxy( $x + $e->{x} - $d,
 		       $y + $e->{y} + $d,
 		       $x + $e->{x} + $e->{width} + $d,
-		       $y - $e->{height} - $d,
-		       0.5, $bgcol, $frame);
-    }
-
-    $x += $e->{width};
-#    print STDERR ("TX: $x\n");
-    return $x;
-}
-
-# Identical copy of text, but without baseline correction.
-sub text_nobl {
-    my ( $self, $text, $x, $y, $font, $size ) = @_;
-#    print STDERR ("T: @_\n");
-    $font ||= $self->{font};
-    $size ||= $font->{size};
-
-    $self->{layout}->set_font_description($font->{fd});
-    $self->{layout}->set_font_size($size);
-    if ( $font->{color} && $font->{color} ne "black" ) {
-	$text = "<span color='" . $font->{color} . "'>" . $text . "</span>";
-    }
-    $self->{layout}->set_markup($text);
-    $self->{layout}->show( $x, $y, $self->{pdftext} );
-
-    my $e = ($self->{layout}->get_pixel_extents)[1];
-
-    # Handle decorations (background, box).
-    my $bgcol = $font->{background};
-    undef $bgcol if $bgcol && $bgcol =~ /^no(?:ne)?$/i;
-    my $debug = "blue";
-    my $frame = $font->{frame} || $debug;
-    undef $frame if $frame && $frame =~ /^no(?:ne)?$/i;
-    if ( $bgcol || $frame ) {
-	#printf("BB: %.2f %.2f %.2f %.2f\n", @{$e}{qw( x y width height ) } );
-	# Draw background and.or frame.
-	my $d = $debug ? 0 : 1;
-	$frame = $debug || $font->{color} || "black" if $frame;
-	$self->rectxy( $x + $e->{x} - $d,
-		       $y + $e->{y} + $d,
-		       $x + $e->{x} + $e->{width} + $d,
-		       $y - $e->{height} - $d,
+		       $y + $e->{y} - $e->{height} - $d,
 		       0.5, $bgcol, $frame);
     }
 
@@ -159,35 +181,33 @@ sub setfont {
     $self->{pdftext}->font( $font->{fd}->{font}, $size );
 }
 
-my $tmplayout;
-
 sub strwidth {
     my ( $self, $text, $font, $size ) = @_;
     $font ||= $self->{font};
     $size ||= $self->{fontsize} || $font->{size};
-    $tmplayout //= Text::Layout->new( $self->{pdf} );
-    $tmplayout->set_font_description($font->{fd});
-    $tmplayout->set_font_size($size);
-    $tmplayout->set_markup($text);
-    $tmplayout->get_pixel_size->{width};
+    $self->{tmplayout} //= Text::Layout->new( $self->{pdf} );
+    $self->{tmplayout}->set_font_description($font->{fd});
+    $self->{tmplayout}->set_font_size($size);
+    $self->{tmplayout}->set_markup($text);
+    $self->{tmplayout}->get_pixel_size->{width};
 }
 
 sub strheight {
     my ( $self, $text, $font, $size ) = @_;
     $font ||= $self->{font};
     $size ||= $self->{fontsize} || $font->{size};
-    $tmplayout //= Text::Layout->new( $self->{pdf} );
-    $tmplayout->set_font_description($font->{fd});
-    $tmplayout->set_font_size($size);
-    $tmplayout->set_markup($text);
-    $tmplayout->get_pixel_size->{height};
+    $self->{tmplayout} //= Text::Layout->new( $self->{pdf} );
+    $self->{tmplayout}->set_font_description($font->{fd});
+    $self->{tmplayout}->set_font_size($size);
+    $self->{tmplayout}->set_markup($text);
+    $self->{tmplayout}->get_pixel_size->{height};
 }
 
 sub line {
     my ( $self, $x0, $y0, $x1, $y1, $lw, $color ) = @_;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
-    $gfx->strokecolor($color ||= "black");
+    $gfx->strokecolor( $self->_fgcolor($color) );
     $gfx->linecap(1);
     $gfx->linewidth($lw||1);
     $gfx->move( $x0, $y0 );
@@ -197,11 +217,12 @@ sub line {
 }
 
 sub hline {
-    my ( $self, $x, $y, $w, $lw, $color ) = @_;
+    my ( $self, $x, $y, $w, $lw, $color, $cap ) = @_;
+    $cap //= 2;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
-    $gfx->strokecolor($color ||= "black");
-    $gfx->linecap(2);
+    $gfx->strokecolor( $self->_fgcolor($color) );
+    $gfx->linecap($cap);
     $gfx->linewidth($lw||1);
     $gfx->move( $x, $y );
     $gfx->hline( $x + $w );
@@ -210,11 +231,12 @@ sub hline {
 }
 
 sub vline {
-    my ( $self, $x, $y, $h, $lw, $color ) = @_;
+    my ( $self, $x, $y, $h, $lw, $color, $cap ) = @_;
+    $cap //= 2;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
-    $gfx->strokecolor($color ||= "black");
-    $gfx->linecap(2);
+    $gfx->strokecolor( $self->_fgcolor($color) );
+    $gfx->linecap($cap);
     $gfx->linewidth($lw||1);
     $gfx->move( $x, $y );
     $gfx->vline( $y - $h );
@@ -226,8 +248,8 @@ sub rectxy {
     my ( $self, $x, $y, $x1, $y1, $lw, $fillcolor, $strokecolor ) = @_;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
-    $gfx->strokecolor($strokecolor) if $strokecolor;
-    $gfx->fillcolor($fillcolor) if $fillcolor;
+    $gfx->strokecolor($self->_fgcolor($strokecolor)) if $strokecolor;
+    $gfx->fillcolor($self->_fgcolor($fillcolor)) if $fillcolor;
     $gfx->linecap(2);
     $gfx->linewidth($lw||1);
     $gfx->rectxy( $x, $y, $x1, $y1 );
@@ -237,12 +259,29 @@ sub rectxy {
     $gfx->restore;
 }
 
+sub poly {
+    my ( $self, $points, $lw, $fillcolor, $strokecolor ) = @_;
+    undef $strokecolor unless $lw;
+    my $gfx = $self->{pdfgfx};
+    $gfx->save;
+    $gfx->strokecolor($self->_fgcolor($strokecolor)) if $strokecolor;
+    $gfx->fillcolor($self->_fgcolor($fillcolor)) if $fillcolor;
+    $gfx->linecap(2);
+    $gfx->linewidth($lw);
+    $gfx->poly( @$points );
+    $gfx->close;
+    $gfx->fill if $fillcolor && !$strokecolor;
+    $gfx->fillstroke if $fillcolor && $strokecolor;
+    $gfx->stroke if $strokecolor && !$fillcolor;
+    $gfx->restore;
+}
+
 sub circle {
     my ( $self, $x, $y, $r, $lw, $fillcolor, $strokecolor ) = @_;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
-    $gfx->strokecolor($strokecolor) if $strokecolor;
-    $gfx->fillcolor($fillcolor) if $fillcolor;
+    $gfx->strokecolor($self->_fgcolor($strokecolor)) if $strokecolor;
+    $gfx->fillcolor($self->_fgcolor($fillcolor)) if $fillcolor;
     $gfx->linewidth($lw||1);
     $gfx->circle( $x, $y, $r );
     $gfx->fill if $fillcolor;
@@ -254,7 +293,7 @@ sub cross {
     my ( $self, $x, $y, $r, $lw, $strokecolor ) = @_;
     my $gfx = $self->{pdfgfx};
     $gfx->save;
-    $gfx->strokecolor($strokecolor) if $strokecolor;
+    $gfx->strokecolor($self->_fgcolor($strokecolor)) if $strokecolor;
     $gfx->linewidth($lw||1);
     $r = 0.9 * $r;
     $gfx->move( $x-$r, $y-$r );
@@ -266,22 +305,42 @@ sub cross {
     $gfx->restore;
 }
 
+sub crosshair {			# for debugging
+    my ( $self, $x, $y, $r, $lw, $strokecolor ) = @_;
+    my $gfx = $self->{pdfgfx};
+    $gfx->save;
+    $gfx->strokecolor($self->_fgcolor($strokecolor)) if $strokecolor;
+    $gfx->linewidth($lw||1);
+    $gfx->move( $x, $y - $r );
+    $gfx->line( $x, $y + $r );
+    $gfx->stroke if $strokecolor;
+    $gfx->move( $x - $r, $y );
+    $gfx->line( $x + $r, $y );
+    $gfx->stroke if $strokecolor;
+    $gfx->restore;
+}
+
 sub get_image {
-    my ( $self, $uri ) = @_;
+    my ( $self, $elt ) = @_;
 
     my $img;
+    my $uri = $elt->{uri};
+    warn("get_image($uri)\n") if $config->{debug}->{images};
     if ( $uri =~ /^id=(.+)/ ) {
 	my $a = $App::Music::ChordPro::Output::PDF::assets->{$1};
-	my $d = $a->{data};
-	my $fh = IO::String->new($d);
-	if ( $a->{type} eq "jpg" ) {
-	    $img = $self->{pdf}->image_jpeg($fh);
+
+	if ( $a->{type} eq "abc" ) {
+	    my $res = App::Music::ChordPro::Output::PDF::abc2image( undef, $self, $a );
+	    return $self->get_image( { %$elt, uri => $res->{src} } );
+	}
+	elsif ( $a->{type} eq "jpg" ) {
+	    $img = $self->{pdf}->image_jpeg(IO::String->new($a->{data}));
 	}
 	elsif ( $a->{type} eq "png" ) {
-	    $img = $self->{pdf}->image_png($fh);
+	    $img = $self->{pdf}->image_png(IO::String->new($a->{data}));
 	}
 	elsif ( $a->{type} eq "gif" ) {
-	    $img = $self->{pdf}->image_gif($fh);
+	    $img = $self->{pdf}->image_gif(IO::String->new($a->{data}));
 	}
 	return $img;
     }
@@ -314,16 +373,71 @@ sub newpage {
     $self->{pdfpage} = $self->{pdf}->page($page||0);
     $self->{pdfpage}->mediabox( $ps->{papersize}->[0],
 				$ps->{papersize}->[1] );
+
+    $self->{pdfgfx}  = $self->{pdfpage}->gfx;
+    $self->{pdftext} = $self->{pdfpage}->text;
+    unless ($ps->{theme}->{background} =~ /^white|none|#ffffff$/i ) {
+	for ( $self->{pdfgfx} ) {
+	    $_->save;
+	    $_->fillcolor( $ps->{theme}->{background} );
+	    $_->linewidth(0);
+	    $_->rectxy( 0, 0, $ps->{papersize}->[0],
+			$ps->{papersize}->[1] );
+	    $_->fill;
+	    $_->restore;
+	}
+    }
+}
+
+sub openpage {
+    my ( $self, $ps, $page ) = @_;
+    $self->{pdfpage} = $self->{pdf}->openpage($page);
     $self->{pdfgfx}  = $self->{pdfpage}->gfx;
     $self->{pdftext} = $self->{pdfpage}->text;
 }
 
+sub importpage {
+    my ( $self, $fn, $pg ) = @_;
+    my $bg = $self->{pdfapi}->open($fn);
+    return unless $bg;		# should have been checked
+    $pg = $bg->pages if $pg > $bg->pages;
+    $self->{pdf}->import_page( $bg, $pg, $self->{pdfpage} );
+    # Make sure the contents get on top of it.
+    $self->{pdfgfx}  = $self->{pdfpage}->gfx;
+    $self->{pdftext} = $self->{pdfpage}->text;
+}
+
+sub importfile {
+    my ( $self, $filename ) = @_;
+    my $pdf = $self->{pdfapi}->open($filename);
+    return unless $pdf;		# should have been checked
+    for ( my $page = 1; $page <= $pdf->pages; $page++ ) {
+	$self->{pdf}->import_page( $pdf, $page );
+    }
+    return { pages => $pdf->pages, $pdf->info_metadata };
+}
+
 sub pagelabel {
     my ( $self, $page, $style, $prefix ) = @_;
-    my $opts = { -style => $style // 'arabic',
-		 defined $prefix ? ( -prefix => $prefix ) : (),
-		 -start => 1 };
-    $self->{pdf}->pageLabel( $page, $opts );
+    $style //= 'arabic';
+
+    # PDF::API2 2.042 has some incompatible changes...
+    my $c = $self->{pdf}->can("page_labels");
+    if ( $c ) {			# 2.042+
+	my $opts = { style => $style eq 'Roman' ? 'R' :
+		              $style eq 'roman' ? 'r' :
+                              $style eq 'Alpha' ? 'A' :
+                              $style eq 'alpha' ? 'a' : 'D',
+		     defined $prefix ? ( prefix => $prefix ) : (),
+		     start => 1 };
+	$c->( $self->{pdf}, $page, $opts );
+    }
+    else {
+	my $opts = { -style => $style,
+		     defined $prefix ? ( -prefix => $prefix ) : (),
+		     -start => 1 };
+	$self->{pdf}->pageLabel( $page, $opts );
+    }
 }
 
 sub make_outlines {
@@ -342,9 +456,18 @@ sub make_outlines {
 	# Seems not to matter whether we re-use the root or create new.
 	$ol_root //= $pdf->outlines;
 
-	my $outline = $ol_root->outline;
-	$outline->title( $ctl->{label} );
-	$outline->closed if $ctl->{collapse};
+	my $outline;
+
+	# Skip level for a single outline.
+	if ( @{ $self->{ps}->{outlines} } == 1 ) {
+	    $outline = $ol_root;
+	    $outline->closed if $ctl->{collapse}; # TODO?
+	}
+	else {
+	    $outline = $ol_root->outline;
+	    $outline->title( $ctl->{label} );
+	    $outline->closed if $ctl->{collapse};
+	}
 
 	my %lh;			# letter hierarchy
 	for ( @$book ) {
@@ -380,7 +503,12 @@ sub make_outlines {
 		}
 		# Display info.
 		$ol->title( demarkup( fmt_subst( $song, $ctl->{line} ) ) );
-		$ol->dest($pdf->openpage( $song->{meta}->{tocpage} + $start ));
+		if ( my $c = $ol->can("destination") ) {
+		    $c->( $ol, $pdf->openpage( $song->{meta}->{tocpage} + $start ) );
+		}
+		else {
+		    $ol->dest($pdf->openpage( $song->{meta}->{tocpage} + $start ));
+		}
 	    }
 	}
     }
@@ -404,12 +532,29 @@ sub init_fonts {
     my $ps = $self->{ps};
     my $fail;
 
-    my $fc = Text::Layout::FontConfig->new;
+    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
 
-    if ( $ps->{fontdir} ) {
-	$fc->add_fontdirs( @{ $ps->{fontdir} } );
+    # Add font dirs.
+    my @d = ( @{$ps->{fontdir}}, ::rsc_or_file("fonts/"), $ENV{FONTDIR} );
+    # Avoid rsc result if dummy.
+    splice( @d, -2, 1 ) if $d[-2] eq "fonts/";
+    for my $fontdir ( @d ) {
+	next unless $fontdir;
+	$fontdir = expand_tilde($fontdir);
+	if ( -d $fontdir ) {
+	    $self->{pdfapi}->can("addFontDirs")->($fontdir);
+	    $fc->add_fontdirs($fontdir);
+	}
+	else {
+	    warn("PDF: Ignoring fontdir $fontdir [$!]\n");
+	    undef $fontdir;
+	}
     }
 
+    # Make sure we have this one.
+    $fc->register_font( "ChordProSymbols.ttf", "chordprosymbols", "", {} );
+
+    # Process the fontconfig.
     foreach my $ff ( keys( %{ $ps->{fontconfig} } ) ) {
 	my @fam = split( /\s*,\s*/, $ff );
 	foreach my $s ( keys( %{ $ps->{fontconfig}->{$ff} } ) ) {
@@ -436,13 +581,13 @@ sub init_font {
     my ( $self, $ff ) = @_;
     my $ps = $self->{ps};
     my $fd;
-    if ( !$fd && $ps->{fonts}->{$ff}->{file} ) {
+    if ( $ps->{fonts}->{$ff}->{file} ) {
 	$fd = $self->init_filefont($ff);
     }
-    if ( !$fd && $ps->{fonts}->{$ff}->{description} ) {
+    elsif ( $ps->{fonts}->{$ff}->{description} ) {
 	$fd = $self->init_pangofont($ff);
     }
-    if ( !$fd && $ps->{fonts}->{$ff}->{name} ) {
+    elsif ( $ps->{fonts}->{$ff}->{name} ) {
 	$fd = $self->init_corefont($ff);
     }
     warn("No font found for \"$ff\"\n") unless $fd;
@@ -455,7 +600,7 @@ sub init_pangofont {
     my $ps = $self->{ps};
     my $font = $ps->{fonts}->{$ff};
 
-    my $fc = Text::Layout::FontConfig->new;
+    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
     eval {
 	$font->{fd} = $fc->from_string($font->{description});
 	$font->{fd}->get_font($self->{layout}); # force load
@@ -473,12 +618,13 @@ sub init_filefont {
     my $ps = $self->{ps};
     my $font = $ps->{fonts}->{$ff};
 
-    my $fc = Text::Layout::FontConfig->new;
+    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
     eval {
-	$font->{fd} = $fc->from_filename($font->{file});
-	$font->{fd}->get_font($self->{layout}); # force load
-	$font->{fd}->{font}->{Name}->{val} =~ s/~.*/~$faketime/ if $regtest;
-	$font->{_ff} = $ff;
+	my $t = $fc->from_filename(expand_tilde($font->{file}));
+	$t->get_font($self->{layout}); # force load
+	$t->{font}->{Name}->{val} =~ s/~.*/~$faketime/ if $regtest;
+	$t->{_ff} = $ff;
+	$font->{fd} = $t;
     };
     $font->{fd};
 }
@@ -488,10 +634,12 @@ sub init_corefont {
 
     my $ps = $self->{ps};
     my $font = $ps->{fonts}->{$ff};
-
-    my $fc = Text::Layout::FontConfig->new;
+    my $cf = App::Music::ChordPro::Output::PDF::is_corefont($font->{name});
+    die("Config error: \"$font->{name}\" is not a built-in font\n")
+      unless $cf;
+    my $fc = Text::Layout::FontConfig->new( debug => $config->{debug}->{fonts} > 1 );
     eval {
-	$font->{fd} = $fc->from_filename($font->{name});
+	$font->{fd} = $fc->from_filename($cf);
 	$font->{fd}->get_font($self->{layout}); # force load
 	$font->{_ff} = $ff;
     };
@@ -506,6 +654,7 @@ sub show_vpos {
 use File::Temp;
 
 my $cname;
+my $rname;
 sub embed {
     my ( $self, $file ) = @_;
     return unless -f $file;
@@ -531,13 +680,21 @@ sub embed {
     $a->movie($cname, "ChordProConfig" );
     $a->open(0);
 
-    open( $cf, '>', $cname );
-    binmode( $cf, ':utf8' );
-    print $cf (::runtimeinfo());
-    close($cf);
+    my $rf;
+    if ( $rname ) {
+	open( $rf, '>', $rname );
+    }
+    else {
+	( $rf, $rname ) = File::Temp::tempfile( UNLINK => 0);
+    }
+    binmode( $rf, ':utf8' );
+    open( $rf, '>', $rname );
+    binmode( $rf, ':utf8' );
+    print $rf (::runtimeinfo());
+    close($rf);
 
     $a = $self->{pdfpage}->annotation();
-    $a->movie($cname, "ChordProRunTime" );
+    $a->movie($rname, "ChordProRunTime" );
     $a->open(0);
 }
 
@@ -545,6 +702,8 @@ END {
     return unless $cname;
     unlink($cname);
     undef $cname;
+    unlink($rname);
+    undef $rname;
 }
 
 1;
